 precision highp float;
 
 varying highp vec2 t_coordinates;
 uniform sampler2D t_data;
 uniform sampler2D t_curve;
 uniform float contrast;
uniform float brightness;
uniform float saturation;

/*
** Contrast, saturation, brightness
** Code of this function is from TGM's shader pack
** http://irrlicht.sourceforge.net/phpBB2/viewtopic.php?t=21057
*/

// For all settings: 1.0 = 100% 0.5=50% 1.5 = 150%
vec3 ContrastSaturationBrightness(vec3 color, float brt, float sat, float con)
{
	// Increase or decrease theese values to adjust r, g and b color channels seperately
	const float AvgLumR = 0.5;
	const float AvgLumG = 0.5;
	const float AvgLumB = 0.5;
	
	const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
	
	vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
	vec3 brtColor = color * brt;
	vec3 intensity = vec3(dot(brtColor, LumCoeff));
	vec3 satColor = mix(intensity, brtColor, sat);
	vec3 conColor = mix(AvgLumin, satColor, con);
	return conColor;
}

 void main()
 {
     lowp vec4 textureColor = texture2D(t_data, t_coordinates);
     lowp vec4 cbsColor = vec4(ContrastSaturationBrightness(textureColor.rgb, brightness, saturation, contrast), textureColor.a);
     
     lowp float redCurveValue = texture2D(t_curve, vec2(cbsColor.r, 0.0)).r;
     lowp float greenCurveValue = texture2D(t_curve, vec2(cbsColor.g, 0.0)).g;
     lowp float blueCurveValue = texture2D(t_curve, vec2(cbsColor.b, 0.0)).b;
     
     gl_FragColor = vec4(redCurveValue, greenCurveValue, blueCurveValue, textureColor.a);
 }