precision highp float;

#define BlendColorDodgef(base, blend) 	((blend == 1.0) ? blend : min(base / (1.0 - blend), 1.0))
#define BlendColorBurnf(base, blend) 	((blend == 0.0) ? blend : max((1.0 - ((1.0 - base) / blend)), 0.0))
#define BlendVividLightf(base, blend) 	((blend < 0.5) ? BlendColorBurnf(base, (2.0 * blend)) : BlendColorDodgef(base, (2.0 * (blend - 0.5))))


// Component wise blending
#define Blend(base, blend, funcf) 		vec3(funcf(base.r, blend.r), funcf(base.g, blend.g), funcf(base.b, blend.b))
#define BlendAdd(base, blend)           (base+blend)
#define DoBlend(base, blend) 		Blend(base, blend, BlendVividLightf)

varying vec2 layer1_coordinates;
varying vec2 base_coordinates;
uniform lowp sampler2D base_texture;
uniform lowp sampler2D layer1_texture;
uniform int blend_mode;
uniform float layer1_opacity;

void main(void) {
	lowp vec4 base_pixel = texture2D(base_texture, base_coordinates);
	lowp vec4 layer1_pixel = texture2D(layer1_texture, layer1_coordinates);
        
	lowp vec3 processing_pixel = DoBlend(base_pixel.rgb, layer1_pixel.rgb);
    lowp float calculated_opacity = layer1_pixel.a * layer1_opacity;
    lowp float alpha_factor = calculated_opacity;

    lowp vec4 processed_pixel = vec4(BlendAdd((1.0-calculated_opacity)*base_pixel.rgb, alpha_factor*processing_pixel.rgb),base_pixel.a*(1.0-calculated_opacity)+base_pixel.a*calculated_opacity);
    

	gl_FragColor = processed_pixel;
}