precision highp float;

varying highp vec2 t_coordinates;
 
uniform sampler2D t_data;
uniform sampler2D t2_data;

 
 // Values from "Graphics Shaders: Theory and Practice" by Bailey and Cunningham
 const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);
//TODO: make a uniform
const lowp float saturation = 0.25;
const lowp float contrast = 1.2;
const lowp float brightness = 0.05;
 
 void main()
 {
      mediump vec4 base = texture2D(t_data, t_coordinates);
     mediump vec4 overlay = texture2D(t2_data, t_coordinates);
     
     mediump float ra;
     if (base.r < 0.5) {
         ra = 2.0 * overlay.r * base.r + base.r * (1.0 - overlay.a);
     } else {
         ra = overlay.a - 2.0 * (1.0 - base.r) * (overlay.a - overlay.r) + base.r * (1.0 - overlay.a);
     }
     
     mediump float ga;
     if (base.g < 0.5) {
         ga = 2.0 * overlay.g * base.g + base.g * (1.0 - overlay.a);
     } else {
         ga = overlay.a - 2.0 * (1.0 - base.g) * (overlay.a - overlay.g) + base.g * (1.0 - overlay.a);
     }
     
     mediump float ba;
     if (base.b < 0.5) {
         ba = 2.0 * overlay.b * base.b + base.b * (1.0 - overlay.a);
     } else {
         ba = overlay.a - 2.0 * (1.0 - base.b) * (overlay.a - overlay.b) + base.b * (1.0 - overlay.a);
     }
     
  lowp vec4 textureColor = vec4(ra, ga, ba, 1.0);

    lowp float luminance = dot(textureColor.rgb, luminanceWeighting);
    lowp vec3 greyScaleColor = vec3(luminance);
    lowp vec4 saturatedColor = vec4(mix(greyScaleColor, textureColor.rgb, saturation), textureColor.w);
    
  lowp vec4 contrastedColor = vec4(((saturatedColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), saturatedColor.w);
  
	gl_FragColor =  vec4((contrastedColor.rgb + vec3(brightness)), contrastedColor.w);

 }